/* dxp_commands.c */

///// INCLUDES
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "dxp_common.h"
#include "dxp_client.h"
#include "dxp_commands.h"

#if defined(__cplusplus) 
#define IMPORTED extern "C" __declspec( dllimport )
#else 
#define IMPORTED __declspec( dllimport )
#endif

///// GLOBALS
tDxpCommandFunction dxpCommandFunctions[] = {
	{eCmnd_set, 		dxpCmndSet},
	{eCmnd_get, 		dxpCmndGet},
	{eCmnd_io, 			dxpCmndIo},
	{eCmnd_keepAlive, 	dxpCmndKeepAlive},
	{eCmnd_rss, 		dxpCmndRss},
	{eCmnd_rcu, 		dxpCmndRcu},
	{eCmnd_pcControl,	dxpCmndPcControl},
	{-1, NULL}
};

///// STATIC GLOBALS (local to file)


///// STATIC FUNCTION PROTOTYPES (optional)


///// STATIC FUNCTION DEFINITIONS (optional)


///// FUNCTION DEFINITIONS
IMPORTED int DxP(char *, int, char *, char *, int, int, int, void *, int, void *, int *);

int processDxpCommand(int cmndClass, int cmndDesc, tDxp *dxpData)
{
	int i, retval;
	
	// let's search for the command class handler to process this DxP command
	i = 0;
	while(dxpCommandFunctions[i].command != -1) {
		// is this the command?
		if(dxpCommandFunctions[i].command == cmndClass) {
			// yes it is, so call it's handler function
			retval = dxpCommandFunctions[i].function(cmndDesc, dxpData);
			break; // don't forget to break out!
		}
		else {
			// nope, so try the next one
			i++;
		}
	}//end while
	
	return retval;
}

int dxpCmndSet(int cmndDesc, tDxp *dxpData)
{
	// NOT YET IMPLEMENTED!
	printf("dxpCmndSet: not yet implemented!\r\n");
	return 0;
}

int dxpCmndGet(int cmndDesc, tDxp *dxpData)
{
	// NOT YET IMPLEMENTED!
	printf("dxpCmndGet: not yet implemented!\r\n");
	return 0;
}

int dxpCmndIo(int cmndDesc, tDxp *dxpData)
{
	// this function implements the eCmnd_io command class
	int i, retval, returnDataSize;
	char returnData[1024];
	char statword[10];
	char c;
	tChangeRelay changeRelay;
	tChangeRelays changeRelays;
	tPulseRelay pulseRelay;
		
	//printf("dxpCmndIo: executing!\r\n");
	
	// zero out the pulse payload and control payload
	memset(&changeRelay, 0, sizeof(tChangeRelay));
	memset(&changeRelays, 0, sizeof(tChangeRelays));
	memset(&pulseRelay, 0, sizeof(tPulseRelay));
	
	// what command descriptor?
	switch(cmndDesc) {
		case eCmndIoDesc_changeRelay:		// change relay 	(0x01)
			changeRelay.channel = dxpData->channel;
			changeRelay.state = dxpData->state;
			retval = DxP(dxpData->ipAddress, dxpData->ipPort, dxpData->username, dxpData->password, eCmnd_io, cmndDesc, dxpData->param, (void *)&changeRelay, sizeof(tChangeRelay), (void *)returnData, &returnDataSize);
			if(retval == DXP_SUCCESS) {
				// DxP() call was successful, but was there a protocol error?
				if(returnData[0] == 1) {
					printf("DxP device error\r\n");
				}
				else {
					// success!
				}
			}
			else {
				// there was an error with DxP()
				printf("DxP() failed with error %d (%s)\r\n", retval, error_text[retval]);
			}
		break;
		
		case eCmndIoDesc_changeRelays:		// change relays	(0x02)
			// need to change ALL of the relays to the same position
			for(i=0; i < 32; i++)
				changeRelays.relayState[i] = dxpData->state;
			
			retval = DxP(dxpData->ipAddress, dxpData->ipPort, dxpData->username, dxpData->password, eCmnd_io, cmndDesc, dxpData->param, (void *)&changeRelays, sizeof(tChangeRelays), (void *)returnData, &returnDataSize);
			if(retval == DXP_SUCCESS) {
				printf("dxpCmndIo::eCmndIoDesc_changeRelays::return data size is %d, returnData[0]=%d\n", returnDataSize, (int)returnData[0]);
			}
			else {
				printf("DxP() failed with error %d (%s)\r\n", retval, error_text[retval]);
			}
		break;
		
		case eCmndIoDesc_getRelay:			// get relay		(0x03)
		
		break;
		
		case eCmndIoDesc_getRelays:			// get relays		(0x04)
			strcpy(statword, "Output ");
			retval = DxP(dxpData->ipAddress, dxpData->ipPort, dxpData->username, dxpData->password, eCmnd_io, cmndDesc, dxpData->param, NULL, 0, (void *)returnData, &returnDataSize);		
			if(retval == DXP_SUCCESS) {
				if(returnDataSize == MAX_RESPONSE_SIZE) {
					// loop through all response bytes
					for(i = 0; i < MAX_RESPONSE_SIZE; i++) {
						c = returnData[i];
						if(!dxpData->sel || (dxpData->sel && (i+1) == dxpData->channel)) {
							if(c == OUTPUT_OPEN) {
								printf("%s %d is Open / Off\r\n", statword, i+1);
							}
							else {
								printf("%s %d is Closed / On\r\n", statword, i+1);
							}
						}
						else {
							// do nothing
						}
					}//end for
				}
				else {
					printf("Did not receive %d bytes from device\r\n", MAX_RESPONSE_SIZE);
				}
			}
			else {
				printf("DxP() failed with error %d (%s)\r\n", retval, error_text[retval]);
			}
		break;
		
		case eCmndIoDesc_getInput:			// get input		(0x05)
		
		break;
		
		case eCmndIoDesc_getInputs:			// get inputs 		(0x06)
			strcpy(statword, "Input ");
			retval = DxP(dxpData->ipAddress, dxpData->ipPort, dxpData->username, dxpData->password, eCmnd_io, cmndDesc, dxpData->param, NULL, 0, (void *)returnData, &returnDataSize);
			if(retval == DXP_SUCCESS) {
				if(returnDataSize == MAX_RESPONSE_SIZE) {
					// loop through all response bytes
					for(i = 0; i < MAX_RESPONSE_SIZE; i++) {
						c = returnData[i];
						if(!dxpData->sel || (dxpData->sel && (i+1) == dxpData->channel)) {
							if(c == INPUT_OPEN) {
								printf("%s %d is Open / Off\r\n", statword, i+1);
							}
							else {
								printf("%s %d is Closed / On\r\n", statword, i+1);
							}
						}
						else {
							// do nothing
						}
					}//end for
				}
				else {
					printf("DxP: did not receive %d bytes from device\r\n", MAX_RESPONSE_SIZE);
				}
			}
			else {
				printf("DxP() failed with error %d (%s)\r\n", retval, error_text[retval]);
			}
		break;
		
		case eCmndIoDesc_pulseRelay:		// pulse relay		(0x07)
			pulseRelay.channel = dxpData->channel;
			pulseRelay.state = dxpData->state;
			pulseRelay.pulseWidth = dxpData->time;
			retval = DxP(dxpData->ipAddress, dxpData->ipPort, dxpData->username, dxpData->password, eCmnd_io, cmndDesc, dxpData->param, (void *)&pulseRelay, sizeof(tPulseRelay), (void *)returnData, &returnDataSize);
			if(retval == DXP_SUCCESS) {
				// DxP() call was successful, but was there a protocol error?
				if(returnData[0] == 1) {
					printf("DxP device error\r\n");
				}
				else {
					// success!
				}
			}
			else {
				// there was an error with DxP()
				printf("DxP() failed with error %d (%s)\r\n", retval, error_text[retval]);
			}
		break;
		
		default:break;
	}//end switch
	
	return retval;
}

int dxpCmndKeepAlive(int cmndDesc, tDxp *dxpData)
{
	// NOT YET IMPLEMENTED!
	printf("dxpCmndKeepAlive: not yet implemented!\r\n");
	return 0;
}

int dxpCmndRss(int cmndDesc, tDxp *dxpData)
{
	// NOT YET IMPLEMENTED!
	printf("dxpCmndRss: not yet implemented!\r\n");
	return 0;
}

int dxpCmndRcu(int cmndDesc, tDxp *dxpData)
{
	// NOT YET IMPLEMENTED!
	printf("dxpCmndRcu: not yet implemented!\r\n");
	return 0;
}

int dxpCmndPcControl(int cmndDesc, tDxp *dxpData)
{
	// NOT YET IMPLEMENTED!
	printf("dxpCmndPcControl: not yet implemented!\r\n");
	return 0;
}


